package net.pterodactylus.sone.database.memory;

import static java.util.Arrays.asList;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.containsInAnyOrder;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.nullValue;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import net.pterodactylus.sone.TestValue;
import net.pterodactylus.util.config.Configuration;
import net.pterodactylus.util.config.ConfigurationException;
import net.pterodactylus.util.config.Value;

import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

/**
 * Unit test for {@link ConfigurationLoader}.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class ConfigurationLoaderTest {

	private final Configuration configuration = mock(Configuration.class);
	private final ConfigurationLoader configurationLoader =
			new ConfigurationLoader(configuration);

	@Test
	public void loaderCanLoadKnownPosts() {
		when(configuration.getStringValue("KnownPosts/0/ID"))
				.thenReturn(TestValue.from("Post2"));
		when(configuration.getStringValue("KnownPosts/1/ID"))
				.thenReturn(TestValue.from("Post1"));
		when(configuration.getStringValue("KnownPosts/2/ID"))
				.thenReturn(TestValue.<String>from(null));
		Set<String> knownPosts = configurationLoader.loadKnownPosts();
		assertThat(knownPosts, containsInAnyOrder("Post1", "Post2"));
	}

	@Test
	public void loaderCanSaveKnownPosts() throws ConfigurationException {
		Map<String, Value<String>> configurationValues = prepareConfigurationValues();
		HashSet<String> originalPosts = new LinkedHashSet<String>(asList("Post1", "Post2"));
		configurationLoader.saveKnownPosts(originalPosts);
		assertThat(configurationValues.get("KnownPosts/0/ID").getValue(), is("Post1"));
		assertThat(configurationValues.get("KnownPosts/1/ID").getValue(), is("Post2"));
		assertThat(configurationValues.get("KnownPosts/2/ID").getValue(), nullValue());
	}

	private Map<String, Value<String>> prepareConfigurationValues() {
		final Map<String, Value<String>> configurationValues = new HashMap<String, Value<String>>();
		when(configuration.getStringValue(anyString())).thenAnswer(new Answer<Value<String>>() {
			@Override
			public Value<String> answer(InvocationOnMock invocation) throws Throwable {
				Value<String> stringValue = TestValue.from(null);
				configurationValues.put((String) invocation.getArguments()[0], stringValue);
				return stringValue;
			}
		});
		return configurationValues;
	}

	@Test
	public void loaderCanLoadKnownPostReplies() {
		when(configuration.getStringValue("KnownReplies/0/ID"))
				.thenReturn(TestValue.from("PostReply2"));
		when(configuration.getStringValue("KnownReplies/1/ID"))
				.thenReturn(TestValue.from("PostReply1"));
		when(configuration.getStringValue("KnownReplies/2/ID"))
				.thenReturn(TestValue.<String>from(null));
		Set<String> knownPosts = configurationLoader.loadKnownPostReplies();
		assertThat(knownPosts,
				containsInAnyOrder("PostReply1", "PostReply2"));
	}

	@Test
	public void loaderCanLoadBookmarkedPosts() {
		when(configuration.getStringValue("Bookmarks/Post/0/ID"))
				.thenReturn(TestValue.from("Post2"));
		when(configuration.getStringValue("Bookmarks/Post/1/ID"))
				.thenReturn(TestValue.from("Post1"));
		when(configuration.getStringValue("Bookmarks/Post/2/ID"))
				.thenReturn(TestValue.<String>from(null));
		Set<String> knownPosts = configurationLoader.loadBookmarkedPosts();
		assertThat(knownPosts, containsInAnyOrder("Post1", "Post2"));
	}

	@Test
	public void loaderCanSaveBookmarkedPosts() throws ConfigurationException {
		final Value<String> post1 = TestValue.<String>from(null);
		final Value<String> post2 = TestValue.<String>from(null);
		final Value<String> post3 = TestValue.<String>from(null);
		when(configuration.getStringValue("Bookmarks/Post/0/ID")).thenReturn(post1);
		when(configuration.getStringValue("Bookmarks/Post/1/ID")).thenReturn(post2);
		when(configuration.getStringValue("Bookmarks/Post/2/ID")).thenReturn(post3);
		HashSet<String> originalPosts = new HashSet<String>(asList("Post1", "Post2"));
		configurationLoader.saveBookmarkedPosts(originalPosts);
		HashSet<String> extractedPosts =
				new HashSet<String>(asList(post1.getValue(), post2.getValue()));
		assertThat(extractedPosts, containsInAnyOrder("Post1", "Post2"));
		assertThat(post3.getValue(), nullValue());
	}

}
